#!/usr/bin/python  
#coding: utf-8
import os
import sys
import psycopg2
from osgeo import ogr
from pyproj import Proj
import Image, ImageDraw

reload(sys)  
sys.setdefaultencoding('utf-8')

def projectLngLatToXY(lng, lat):
	return project(lng, lat)

def convertXYToPixel(x, y, w, h, minx, maxx, miny, maxy):
	x1 = w * (x - minx) / (maxx - minx)
	y1 = h * (maxy- y) / (maxy - miny)
	return (x1, y1)

def drawPolygon(geom, draw, w, h, minx, maxx, miny, maxy):
	for i in range(0, geom.GetGeometryCount()):
		g = geom.GetGeometryRef(i)
		pts = []
		for j in range(0, g.GetPointCount()):
			pt = g.GetPoint(j)
			ptj = projectLngLatToXY(pt[0], pt[1])
			point = convertXYToPixel(ptj[0], ptj[1], w, h, minx, maxx, miny, maxy)
			pts.append(point)
		draw.polygon(pts, outline='#000000', fill='#5f9ea0')
	return

connection = psycopg2.connect(database="foss4g", user="postgres", password="escp@2010", host="162.105.17.181", port="5432")
cursor = connection.cursor()

#CREATE TABLE
print 'CREATE TABLE......'
cursor.execute("DROP TABLE IF EXISTS public.china_provinces;")
cursor.execute("CREATE TABLE public.china_provinces (gid int4 NOT NULL primary key, name varchar(30), popname varchar(30), code varchar(9), provinceid varchar(20), geom public.geometry) WITH (OIDS=FALSE); CREATE INDEX  china_provinces_geom_idx ON public.china_provinces USING gist(geom);")
connection.commit()
print 'COMPLETED.'

#READ SHAPEFILE AND INSERT GEOMETRY INTO TABLE
print 'IMPORT SHAPEFILE INTO POSTGIS......'
shapefile = ogr.Open('province_region.shp') 
layer = shapefile.GetLayer(0)    
for i in range(layer.GetFeatureCount()):  
	feature = layer.GetFeature(i)  
	name = feature.GetField("NAME").decode("utf-8")
	popname = feature.GetField("POPNAME").decode("utf-8")
	code = feature.GetField("CODE").decode("utf-8")  
	provinceid = feature.GetField("ID").decode("utf-8")  
	wkt = feature.GetGeometryRef().ExportToWkt()
	cursor.execute("INSERT INTO public.china_provinces (gid, name, popname, code, provinceid, geom) " +"VALUES (%s, %s, %s, %s, %s, ST_GeometryFromText(%s, " +"4326))", (i + 1, name, popname, code, provinceid, wkt))
connection.commit()
print 'COMPLETED.'

#READ GEOMETRY FROM POSTGIS AND DRAW
print 'EXPORT POSTGIS TO A IMAGE......'
cursor.execute("SELECT ST_XMin(bbox.geom), ST_XMax(bbox.geom), ST_YMin(bbox.geom), ST_YMax(bbox.geom) FROM (SELECT ST_Envelope(ST_Union(geom)) AS geom FROM china_provinces) AS bbox")
bbox = cursor.fetchone()
minlng = bbox[0]
maxlng = bbox[1]
minlat = bbox[2]
maxlat = bbox[3]
project = Proj('+proj=aea +lon_0=%f +lat_1=%f +lat_2=%f +ellps=krass' % ((minlng + maxlng) / 2, minlat, maxlat))
p1 = projectLngLatToXY(minlng, minlat)
p2 = projectLngLatToXY(maxlng, maxlat)
minx = p1[0]
miny = p1[1]
maxx = p2[0]
maxy = p2[1]
print 'CHINA_PROVINCES BOUNDING: %s, %s, %s, %s' % (minlng, minlat, maxlng, maxlat)
print 'CHINA_PROVINCES BOUNDING: %s, %s, %s, %s' % (minx, miny, maxx, maxy)

##CREATE NEW IMAGE
print 'CREATE NEW IMAGE......'
w = 1280
h = int(w * (maxy - miny) / (maxx - minx))
print 'CREATE export.png: %s * %s' % (w, h)
img = Image.new("RGBA", (1500, 1200), (255,255,255))
draw = ImageDraw.Draw(img)

print 'SELECT GEOMETRY FROM POSTGIS AND DRAW A IMAGE......'
cursor.execute("SELECT ST_ASTEXT(geom) FROM public.china_provinces ORDER BY gid")
provinces = cursor.fetchall()
for province in provinces:
	geometry = ogr.CreateGeometryFromWkt(province[0])
	gtype = geometry.GetGeometryName()
	if gtype == 'MULTIPOLYGON':
		for i in range(0, geometry.GetGeometryCount()):
			g = geometry.GetGeometryRef(i)
			drawPolygon(g, draw, w, h, minx, maxx, miny, maxy)
	else:
		drawPolygon(geometry, draw, w, h, minx, maxx, miny, maxy)
del draw
img.save('export.png', 'PNG')
print 'COMPLETED.'
cursor.close()
connection.close()