#!/usr/bin/python  
#coding: utf-8
import os
import sys
import psycopg2
from osgeo import ogr
import pyproj
#import Image, ImageDraw
from PIL import Image
from PIL import ImageDraw
from pyproj import Proj


#reload(sys)  
#sys.setdefaultencoding('utf-8')

def convertLngLatToXY(lng, lat, w, h, minlng, maxlng, minlat, maxlat):
	x = w * (lng - minlng) / (maxlng - minlng)
	y = h * (maxlat - lat) / (maxlat - minlat)
	return (x, y)

def drawPolygon(geom, draw, w, h, minlng, maxlng, minlat, maxlat):
	for i in range(0, geom.GetGeometryCount()):
		g = geom.GetGeometryRef(i)
		pts = []
		for j in range(0, g.GetPointCount()):
			pt = g.GetPoint(j)
			pt = Merc(pt[0],pt[1])
			point = convertLngLatToXY(pt[0], pt[1], w, h, minlng, maxlng, minlat, maxlat)
			pts.append(point)
		draw.polygon(pts, outline='#000000', fill='#5f9ea0')
	return

connection = psycopg2.connect(database="ccdb", user="postgres", password="postgres", host="127.0.0.1", port="5432")
cursor = connection.cursor()

#CREATE TABLE
print('CREATE TABLE......')
cursor.execute("DROP TABLE IF EXISTS public.CHN_adm2;")
cursor.execute("CREATE TABLE public.CHN_adm2 (gid int4 NOT NULL primary key, geom public.geometry) WITH (OIDS=FALSE); CREATE INDEX  CHN_adm2_geom_idx ON public.CHN_adm2 USING gist(geom);")
connection.commit()
print('COMPLETED.')

#READ SHAPEFILE AND INSERT GEOMETRY INTO TABLE
print('IMPORT SHAPEFILE INTO POSTGIS......')
shapefile = ogr.Open('/home/momo/下载/CHN_adm/CHN_adm1.shp')
layer = shapefile.GetLayer(0)    
for i in range(layer.GetFeatureCount()):  
	feature = layer.GetFeature(i)  
	#name = feature.GetField("NAME").decode("utf-8")
	#popname = feature.GetField("POPNAME").decode("utf-8")
	#code = feature.GetField("CODE").decode("utf-8")  
	#provinceid = feature.GetField("ID").decode("utf-8")  
	wkt = feature.GetGeometryRef().ExportToWkt()
	cursor.execute("INSERT INTO public.CHN_adm2 (gid, geom) " +"VALUES (%s, ST_GeometryFromText(%s, " +"4326))", (i + 1, wkt))
connection.commit()
print('COMPLETED.')

#READ GEOMETRY FROM POSTGIS AND DRAW
print('EXPORT POSTGIS TO A IMAGE......')
cursor.execute("SELECT ST_XMin(bbox.geom), ST_XMax(bbox.geom), ST_YMin(bbox.geom), ST_YMax(bbox.geom) FROM (SELECT ST_Envelope(ST_Union(geom)) AS geom FROM public.CHN_adm2) AS bbox")
bbox = cursor.fetchone()
minlng = bbox[0]
maxlng = bbox[1]
minlat = bbox[2]
maxlat = bbox[3]
print('CHINA_PROVINCES BOUNDING: %s, %s, %s, %s' % (minlng, minlat, maxlng, maxlat))

#CREATE NEW IMAGE
print('CREATE NEW IMAGE......')
Merc = pyproj.Proj("+init=EPSG:3857")
minlng,minlat = Merc(minlng,minlat)
maxlng,maxlat = Merc(maxlng,maxlat)
w = 1280
h = int(w * (maxlat - minlat) / (maxlng - minlng))
print('CREATE export.png: %s * %s' % (w, h))
img = Image.new("RGBA", (w, h), (255,255,255))
draw = ImageDraw.Draw(img)

print('SELECT GEOMETRY FROM POSTGIS AND DRAW A IMAGE......')
cursor.execute("SELECT ST_ASTEXT(geom) FROM public.CHN_adm2 ORDER BY gid")
provinces = cursor.fetchall()
for province in provinces:
	geometry = ogr.CreateGeometryFromWkt(province[0])
	gtype = geometry.GetGeometryName()
	if gtype == 'MULTIPOLYGON':
		for i in range(0, geometry.GetGeometryCount()):
			g = geometry.GetGeometryRef(i)
			drawPolygon(g, draw, w, h, minlng, maxlng, minlat, maxlat)
	else:
		drawPolygon(geometry, draw, w, h, minlng, maxlng, minlat, maxlat)
del draw
img.save('export.png', 'PNG')
print('COMPLETED.')
cursor.close()
connection.close()
